@@ -138,8 +138,8 @@ package Sidef::Types::Range::RangeNumber {
 
     sub sum_by {
         my ($self, $block) = @_;
-
-        $self->_sum_prod_by('sum', Sidef::Types::Number::Number::ZERO, sub { $block->run($_[0]) },);
+        $block //= Sidef::Types::Block::Block::IDENTITY;
+        $self->_sum_prod_by('sum', Sidef::Types::Number::Number::ZERO, sub { $block->run($_[0]) });
     }
 
     sub sum {
@@ -166,10 +166,25 @@ package Sidef::Types::Range::RangeNumber {
         $n->inc->mul($z->mul($n)->add($x->mul($two)))->div($two);
     }
 
+    sub avg_by {
+        my ($self, $block) = @_;
+        $self->sum_by($block)->div($self->len);
+    }
+
+    sub avg {
+        my ($self, $arg) = @_;
+
+        if (defined($arg)) {
+            goto &avg_by;
+        }
+
+        $self->sum->div($self->len);
+    }
+
     sub prod_by {
         my ($self, $block) = @_;
-
-        $self->_sum_prod_by('prod', Sidef::Types::Number::Number::ONE, sub { $block->run($_[0]) },);
+        $block //= Sidef::Types::Block::Block::IDENTITY;
+        $self->_sum_prod_by('prod', Sidef::Types::Number::Number::ONE, sub { $block->run($_[0]) });
     }
 
     sub prod {
