@@ -404,18 +404,41 @@ package Sidef::Types::Set::Bag {
     sub map {
         my ($self, $block) = @_;
 
+        $block //= Sidef::Types::Block::Block::IDENTITY;
+
         my %new;
         foreach my $key (CORE::keys(%$self)) {
 
-            my $elem  = $self->{$key};
-            my $value = $block->run($elem->{value});
+            my $elem = $self->{$key};
 
-            ($new{$serialize->($value)} //= {value => $value})->{count} += $elem->{count};
+            foreach my $value ($block->run($elem->{value})) {
+                ($new{$serialize->($value)} //= {value => $value})->{count} += $elem->{count};
+            }
         }
 
         bless \%new, ref($self);
     }
 
+    sub map_2d {
+        my ($self, $block) = @_;
+
+        $block //= Sidef::Types::Block::Block::ARRAY_IDENTITY;
+
+        my %new;
+        foreach my $key (CORE::keys(%$self)) {
+
+            my $elem = $self->{$key};
+
+            foreach my $value ($block->run(@{$elem->{value}})) {
+                ($new{$serialize->($value)} //= {value => $value})->{count} += $elem->{count};
+            }
+        }
+
+        bless \%new, ref($self);
+    }
+
+    *map_2D = \&map_2d;
+
     sub collect {
         my ($self, $block) = @_;
 
@@ -430,6 +453,8 @@ package Sidef::Types::Set::Bag {
     sub grep {
         my ($self, $block) = @_;
 
+        $block //= Sidef::Types::Block::Block::IDENTITY;
+
         my %new;
         foreach my $key (CORE::keys(%$self)) {
 
@@ -449,6 +474,30 @@ package Sidef::Types::Set::Bag {
 
     *select = \&grep;
 
+    sub grep_2d {
+        my ($self, $block) = @_;
+
+        $block //= Sidef::Types::Block::Block::ARRAY_IDENTITY;
+
+        my %new;
+        foreach my $key (CORE::keys(%$self)) {
+
+            my $elem  = $self->{$key};
+            my $value = $elem->{value};
+
+            if ($block->run(@$value)) {
+                $new{$key} = {
+                              value => $value,
+                              count => $elem->{count},
+                             };
+            }
+        }
+
+        bless \%new, ref($self);
+    }
+
+    *grep_2D = \&grep_2d;
+
     sub count_by {
         my ($self, $block) = @_;
 
@@ -581,6 +630,21 @@ package Sidef::Types::Set::Bag {
         $self;
     }
 
+    sub each_2d {
+        my ($self, $block) = @_;
+
+        foreach my $elem (CORE::values(%$self)) {
+            my $value = $elem->{value};
+            foreach my $i (1 .. $elem->{count}) {
+                $block->run(@$value);
+            }
+        }
+
+        $self;
+    }
+
+    *each_2D = \&each_2d;
+
     sub each_kv {
         my ($self, $block) = @_;
 
